# coding: utf-8
'''
GRANT

Assigns privileges to roles on database resources, such as keyspaces, tables, functions.

**Important:** Permissions apply immediately, even to active client sessions.

Synopsis


GRANT permission
  [ ON object ]
  TO role_name ;


**Note:** Enclose the role name in single quotation marks if it contains special characters or capital letters.

**Note:** To modify a base table that has a materialized view (MV) using an `INSERT` or `UPDATE` command if access permissions are enabled, a user must be granted `MODIFY` or `ALL PERMISSIONS` on the base table.

privilege
    Permissions granted on a resource to a role; grant a privilege at any level of the resource hierarchy. The full set of available privileges is:

    -   ALL PERMISSIONS
    -   ALTER
    -   AUTHORIZE
    -   CREATE
    -   DESCRIBE
    -   DROP
    -   EXECUTE
    -   MODIFY
    -   PROXY.EXECUTE
    -   PROXY.LOGIN
    -   SEARCH.ALTER
    -   SEARCH.COMMIT
    -   SEARCH.CREATE
    -   SEARCH.DROP
    -   SEARCH.REBUILD
    -   SEARCH.RELOAD
    -   SELECT

resource_name
    The DataStax EnterpriseDataStax Distribution of Apache Cassandra™ database objects to which permissions are applied. Database resources have modelled hierarchy. Grant permissions on a resource higher in the chain to automatically grant that same permission on all resources lower down.

    **Note:** Not all privileges apply to every type of resource. For instance, `EXECUTE` is only relevant in the context of functions, MBeans, RPC, and authentication schemes. Attempting to grant privileges on a resource that the permission is not applicable results in an error.

Functions
    -   ALL FUNCTIONS
    -   ALL FUNCTIONS IN KEYSPACE keyspace_name
    -   FUNCTION keyspace_name.function_name

Data
    -   ALL KEYSPACES
    -   TABLE table_name
    -   Rows ('filtering_data' ROWS IN table_name)
    -   *Indexes* (other than search indexes) belong to a table but permission cannot be directly assigned. ALTER permission on a table allows users to CREATE and DROP indexes.
    -   KEYSPACE keyspace_name
    -   *Types* belong to a keyspace but permissions cannot be directly assigned. To manage types set the keyspace permissions to CREATE, DROP, or ALTER which also gives the user the same permissions for tables.

Search index
    -   ALL SEARCH INDICES
    -   SEARCH KEYSPACE keyspace_name
    -   SEARCH INDICES [keyspace_name.]table_name

JMX
    -   ALL MBEANS
        -   MBEAN mbean_name
        -   MBEANS pattern

Roles
    -   ALL ROLES
    -   ROLE role_name

        **Note:** When using the `internal` Role Management mode nest roles using `GRANT role_name TO role_name` to give all the permissions of the first role in the statement to the second role. Use roles to create your own hierarchical permissions structures.


Remote procedure calls (RPC)
    -   ALL OBJECTS
    -   OBJECT object_name
    -   METHOD

Authentication schemes
    -   ALL SCHEMES
    -   (LDAP | KERBEROS | INTERNAL) SCHEME

Spark applications
    Submit applications:

    -   ANY WORKPOOL
    -   WORKPOOL datacenter_name

    Manage applications:

    -   ANY SUBMISSION
    -   ANY SUBMISSION IN WORKPOOL datacenter_name
    -   SUBMISSION application_ID IN WORKPOOL datacenter_name


'''
