# coding: utf-8
'''
SELECT

Returns data from a single table. A select statement without a where clause returns all rows from all partitions, but is not recommended.

CAUTION:

DataStax recommends limiting queries to a single partition using the WHERE clause. Queries across multiple partitions can impact performance.

Synopsis


SELECT selectors 
  FROM [keyspace_name.]table_name 
  [ WHERE [ primary_key_conditions ] [ AND ] [ index_conditions ]
  [ GROUP BY column_name [ , ... ] ]
  [ ORDER BY PK_column_name [ , ... ] ( ASC | DESC ) ] 
  [ ( LIMIT N | PER PARTITION LIMIT N ) ]
  [ ALLOW FILTERING ] ;


selectors

Determines the columns returned in the results set.


column_list | DISTINCT partition_key [ AS output_name ] 


**Restriction:** Use either a column list or DISTINCT partition_key.

column_list
    Determines the columns and column order returned in the result set. Specify a comma separated list of columns or use an asterisk to return all columns in the stored order.

    
    column_name | function_name( argument_list )
    

    -   column_name: Includes a column in result set.
    -   function_name( arguments ): Execute a function on the specified argument for each row in the result set, see CQL native functions and Creating user-defined function (UDF).
    -   aggregate_name( arguments ): Executes the aggregate on matching data and returns a single result, see CQL native aggregates and CREATE AGGREGATE.

DISTINCT partition_key
    Returns unique values for the full partition key. Use a comma-separated list for compound partition keys.

AS output_name
    Renames the column to the new output name in the result set; for example:

    
    count(id) AS "Cyclist Count"
    

    **Note:** If the name contains special character, spaces, or to retain capitalization, surround the new name with double quotes.

keyspace_name.table_name

The keyspace name is required to identify a table in a different keyspace or if no keyspace is set for the session. If the keyspace or table name contain uppercase letters, enclose the name in double quotes; for example:


FROM "TestTable"


primary_key_conditions

Improves the efficiency of the query using logic statements to identify the data location and allows filtering on the last clustering column.


[ partition_conditions 
  [ AND clustering_conditions ] | [ AND index_conditions ]


**Tip:** To return all the data stored on a partition only define the partition key values.

Logical statement syntax

column_name
    Enclose column names that have uppercase or special characters in double quotes.

    **Note:** Enclose string values in single quotes.

operators
    DataStax supports the following operators:

value
    Enclose string values in single quotes.

    **Note:** Enclose column names that have uppercase or special characters in double quotes.

Identifying the data location and filtering by clustering columns

Use WHERE clauses to maximize read efficiency by identifying the location of the data. The database evaluates the WHERE logical statements hierarchically:

1.  Partition key columns: Use the equal operator to identify all partition key values (or none). Ensure that the data model supports single partition queries to avoid performance issues.

    **Note:** Partitions are typically large sets of data. The partitioner distributes the data by creating a hash of the partition key columns and stores all the rows with the same hash on the same node. Similar or like data, such as partition key date column values 7/01/2017 and 7/02/2017, may not be located on the same node.

2.  Clustering columns determines the sort order within the partition; data is sorted by the first clustering column, the second and so forth.

**Note:** ALLOW FILTERING overrides restrictions on filtering partition, clustering, and regular columns, but can negatively impact performance by causing read latencies. In test environments use cqlsh TRACING.

partition_conditions
    The database requires that all partitions are restricted except when querying a secondary or search index. Use logic statements that identify the partition key columns with following operators:

    -   **Equals (=)**: Any partition key column.
    -   **IN**: Restricted to the last column of the partition key to search multiple partitions.
    -   **Range (>=, <=, >, and <) on tokens**: Fully tokenized partition key (all PK columns specified in order as arguments of the TOKEN function). Use token ranges to scan data stored on a particular node.

    **Note:** For secondary index queries, equals is the only operator supported for partition key logical statements.

    See Partition keys for usage examples and instructions.

clustering_conditions
    Use logic statements that identify the clustering segment. Clustering columns set the sort order of the stored data, which is nested when there are multiple clustering columns. After evaluating the partition key, the database evaluates the clustering statements in the nested order, the first (top level), second, third, and so on.

    All operators are supported in logical statements if the table has only one clustering column. To efficiently locate the data within the partition for tables with multiple clustering columns, the following restrictions apply:

    -   Top level clustering columns:
        -   Equals (=)
        -   IN
    -   Last clustering column statement: All operators and multi-column comparisons

    Clustering column logic statements also support returning slices across clustering segments:

    
    ( column1, column2, ... ) operator ( value1, value2, ... )
    [ AND ( column1, column2, ... ) operator ( value1, value2, ... ) ]
    

    The slice identifies the row that has the corresponding values and allows you to return all rows before, after, or between (when two slice statements are included).

    See Clustering columns for usage examples and instructions.

index_conditions

The DataStax Enterprise (DSE) database supports three types of indexes.DataStax Distribution of Apache Cassandra™ (DDAC) database supports one type of index.

secondary index
    Logical statement syntax for secondary index columns:

    
    column_name operator value
    

    where an operator is:

    -   =
    -   CONTAINS on index collection types
    -   CONTAINS KEY on index map types

Solr query
    The query expression uses the syntax supported by the Solr q parameter. See Writing a basic index query with CQL query syntax and Writing advanced solr_query expressions JSON-based expressions.

SASI index
    To retrieve data using a SSTable Attached Secondary Index, see Using SASI.

Additional options

Change the scope and order of the data returned by the query.

GROUP BY column_name
    Return unique values of the target column name.

ORDER BY (ASC | DESC)
    Sorts the result set in either ascending (ASC) or descending (DESC) order.

    **Note:** When no order is specified the results are returned in the ordered that they are stored.

ALLOW FILTERING
    Enables filtering without including logic statements that the identify primary key or allows filtering on primary keys.

    **Note:** For more information, see Allow Filtering explained.

LIMIT *N* | PER PARTITION LIMIT *N*
    Limits the number of records returned in the results set.


'''
